@@ -39,6 +39,10 @@ int faiss_IndexPreTransform_new_with_transform(
         FaissVectorTransform* ltrans,
         FaissIndex* index);
 
+int faiss_IndexPreTransform_prepend_transform(
+        FaissIndexPreTransform* index,
+        FaissVectorTransform* ltrans);
+
 #ifdef __cplusplus
 }
 #endif
