@@ -11,9 +11,9 @@
 #ifndef FAISS_DEVICE_UTILS_C_H
 #define FAISS_DEVICE_UTILS_C_H
 
-#include <cuda_runtime_api.h>
 #include <cublas.h>
-#include "faiss_c.h"
+#include <cuda_runtime_api.h>
+#include "../faiss_c.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -32,7 +32,6 @@ int faiss_gpu_profiler_stop();
 /// cudaDeviceSynchronize for each device)
 int faiss_gpu_sync_all_devices();
 
-
 #ifdef __cplusplus
 }
 #endif
