@@ -12,10 +12,8 @@
 
 #include <faiss/IndexBinary.h>
 
-
 namespace faiss {
 
-
 struct Index;
 
 /** IndexBinary backed by a float Index.
@@ -26,27 +24,30 @@ struct Index;
  * vectors.
  */
 struct IndexBinaryFromFloat : IndexBinary {
-  Index *index = nullptr;
+    Index* index = nullptr;
 
-  bool own_fields = false; ///< Whether object owns the index pointer.
+    bool own_fields = false; ///< Whether object owns the index pointer.
 
-  IndexBinaryFromFloat();
+    IndexBinaryFromFloat();
 
-  explicit IndexBinaryFromFloat(Index *index);
+    explicit IndexBinaryFromFloat(Index* index);
 
-  ~IndexBinaryFromFloat();
+    ~IndexBinaryFromFloat();
 
-  void add(idx_t n, const uint8_t *x) override;
+    void add(idx_t n, const uint8_t* x) override;
 
-  void reset() override;
+    void reset() override;
 
-  void search(idx_t n, const uint8_t *x, idx_t k,
-              int32_t *distances, idx_t *labels) const override;
+    void search(
+            idx_t n,
+            const uint8_t* x,
+            idx_t k,
+            int32_t* distances,
+            idx_t* labels) const override;
 
-  void train(idx_t n, const uint8_t *x) override;
+    void train(idx_t n, const uint8_t* x) override;
 };
 
+} // namespace faiss
 
-}  // namespace faiss
-
-#endif  // FAISS_INDEX_BINARY_FROM_FLOAT_H
+#endif // FAISS_INDEX_BINARY_FROM_FLOAT_H
