@@ -29,7 +29,7 @@ IndexScalarQuantizer::IndexScalarQuantizer(
         int d,
         ScalarQuantizer::QuantizerType qtype,
         MetricType metric)
-        : Index(d, metric), sq(d, qtype) {
+        : IndexFlatCodes(0, d, metric), sq(d, qtype) {
     is_trained = qtype == ScalarQuantizer::QT_fp16 ||
             qtype == ScalarQuantizer::QT_8bit_direct;
     code_size = sq.code_size;
@@ -43,13 +43,6 @@ void IndexScalarQuantizer::train(idx_t n, const float* x) {
     is_trained = true;
 }
 
-void IndexScalarQuantizer::add(idx_t n, const float* x) {
-    FAISS_THROW_IF_NOT(is_trained);
-    codes.resize((n + ntotal) * code_size);
-    sq.compute_codes(x, &codes[ntotal * code_size], n);
-    ntotal += n;
-}
-
 void IndexScalarQuantizer::search(
         idx_t n,
         const float* x,
@@ -67,6 +60,7 @@ void IndexScalarQuantizer::search(
         InvertedListScanner* scanner =
                 sq.select_InvertedListScanner(metric_type, nullptr, true);
         ScopeDeleter1<InvertedListScanner> del(scanner);
+        scanner->list_no = 0; // directly the list number
 
 #pragma omp for
         for (idx_t i = 0; i < n; i++) {
@@ -99,27 +93,7 @@ DistanceComputer* IndexScalarQuantizer::get_distance_computer() const {
     return dc;
 }
 
-void IndexScalarQuantizer::reset() {
-    codes.clear();
-    ntotal = 0;
-}
-
-void IndexScalarQuantizer::reconstruct_n(idx_t i0, idx_t ni, float* recons)
-        const {
-    std::unique_ptr<ScalarQuantizer::Quantizer> squant(sq.select_quantizer());
-    for (size_t i = 0; i < ni; i++) {
-        squant->decode_vector(&codes[(i + i0) * code_size], recons + i * d);
-    }
-}
-
-void IndexScalarQuantizer::reconstruct(idx_t key, float* recons) const {
-    reconstruct_n(key, 1, recons);
-}
-
 /* Codec interface */
-size_t IndexScalarQuantizer::sa_code_size() const {
-    return sq.code_size;
-}
 
 void IndexScalarQuantizer::sa_encode(idx_t n, const float* x, uint8_t* bytes)
         const {
