@@ -161,9 +161,6 @@ void NSG::search(
     search_on_graph<false>(
             *final_graph, dis, vt, enterpoint, pool_size, retset, tmp);
 
-    std::partial_sort(
-            retset.begin(), retset.begin() + k, retset.begin() + pool_size);
-
     for (size_t i = 0; i < k; i++) {
         I[i] = retset[i].id;
         D[i] = retset[i].distance;
