@@ -3,10 +3,7 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-# translation of test_meta_index.lua
-
+import os
 import sys
 import numpy as np
 import faiss
@@ -78,6 +75,8 @@ class IDRemap(unittest.TestCase):
 
 class Shards(unittest.TestCase):
 
+    @unittest.skipIf(os.name == "posix" and os.uname().sysname == "Darwin",
+                     "There is a bug in the OpenMP implementation on OSX.")
     def test_shards(self):
         k = 32
         ref_index = faiss.IndexFlatL2(d)
@@ -87,13 +86,6 @@ class Shards(unittest.TestCase):
         _Dref, Iref = ref_index.search(xq, k)
         print(Iref[:5, :6])
 
-        # there is a OpenMP bug in this configuration, so disable threading
-        if sys.platform == "darwin" and "Clang 12" in sys.version:
-            nthreads = faiss.omp_get_max_threads()
-            faiss.omp_set_num_threads(1)
-        else:
-            nthreads = None
-
         shard_index = faiss.IndexShards(d)
         shard_index_2 = faiss.IndexShards(d, True, False)
 
@@ -139,8 +131,6 @@ class Shards(unittest.TestCase):
             print('%d / %d differences' % (ndiff, nq * k))
             assert(ndiff < nq * k / 1000.)
 
-        if nthreads is not None:
-            faiss.omp_set_num_threads(nthreads)
 
 class Merge(unittest.TestCase):
 
